
import { LanguageSupport } from "@codemirror/language"
import { completeFromList, ifIn, ifNotIn, snippetCompletion, snippet, closeBrackets } from '@codemirror/autocomplete'
import language from "./language.js"


/**
 * 
 * @param {import("@codemirror/autocomplete").CompletionSource[]} sources 
 */
const completions = sources => sources.map(source => language.data.of({ autocomplete: source }))
export const verilogCompletion = [
    ...completions([
        ifNotIn([
                'Description', 
                'LineComment', 
                'BlockComment',
                'DocComment',
                'String'
            ], 
            completeFromList([
                snippetCompletion(
                    'module ${module_name} (${});\n\t${}\nendmodule',
                    { label: 'module', type: 'class' }
                ),
                snippetCompletion(
                    'module ${module_name} #(${}) (${});\n\t${}\nendmodule',
                    { label: 'module', type: 'class', detail: 'with parameters' }
                ),
                snippetCompletion(
                    'module ${0}_tb;\n\treg clock = 0;\n\treg reset = 1;\n\t${1}\n\n\t'
                    + '${0} DUT ();\n\n\talways #5 clock <= !clock;\nendmodule',
                    { label: 'module', detail: 'testbench' }
                )
            ])),
        ifIn(['ModuleItemList'], 
            ifNotIn([
                'SpecifyBlock',
                'SeqBlock',
                'ParBlock'
            ], 
            completeFromList([
                { label: 'parameter', type: 'variable', 
                    apply: snippet('parameter ${0} = ${1:0};') },
                { label: 'always', type: 'method', detail: 'sequential', 
                    apply: snippet('always @(posedge ${clock}) begin\n\t${}\nend') },
                { label: 'always', type: 'method', detail: 'sync. reset', 
                    apply: snippet('always @(posedge ${clock})\nif (${reset}) begin\n\t\nend else begin\n\t\nend') },
                { label: 'always', type: 'method', detail: 'combinational',
                    apply: snippet('always @(${*}) begin\n\t${}\nend') },
                { label: 'reg', type: 'variable' },
                { label: 'wire', type: 'variable' },
            ])
            )),
        ifIn(['ModuleParameterPortList'], completeFromList([ 
            snippetCompletion('parameter ${0} = ${1:0}',
                { label: 'parameter', type: 'variable' })
        ])),
        completeFromList([
            { label: 'begin', type: 'keyword', apply: snippet('begin\n\t${}\nend') },
            { label: 'begin', type: 'keyword', detail: 'with label', apply: snippet('begin : ${}\n\t${}\nend') }
        ])
    ])
];

export function verilog() {
    return new LanguageSupport(
        language, 
        [
            verilogCompletion,
            closeBrackets()
        ]);
}